@@ -1,6 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import print_function
-
 import asyncore
 import sys
 from logging import getLogger
@@ -15,6 +13,7 @@ logger = getLogger(__name__)
 
 class ExtensionDebuggingServer(SMTPServer):
     """Duplication of smtpd.DebuggingServer, but using logging instead of print."""
+
     # Do something with the gathered message
     def process_message(self, peer, mailfrom, rcpttos, data, **kwargs):
         """Output will be sent to the module logger at INFO level."""
@@ -37,7 +36,7 @@ class Command(BaseCommand):
     requires_system_checks = False
 
     def add_arguments(self, parser):
-        super(Command, self).add_arguments(parser)
+        super().add_arguments(parser)
         parser.add_argument(
             '--output', dest='output_file', default=None,
             help='Specifies an output file to send a copy of all messages (not flushed immediately).'
@@ -77,10 +76,7 @@ class Command(BaseCommand):
         def inner_run():
             quit_command = (sys.platform == 'win32') and 'CTRL-BREAK' or 'CONTROL-C'
             print("Now accepting mail at %s:%s -- use %s to quit" % (addr, port, quit_command))
-            if sys.version_info < (3, 5):
-                ExtensionDebuggingServer((addr, port), None)
-            else:
-                ExtensionDebuggingServer((addr, port), None, decode_data=True)
+            ExtensionDebuggingServer((addr, port), None, decode_data=True)
             asyncore.loop()
 
         try:
