@@ -3,9 +3,20 @@ import json
 import os
 import re
 from distutils.version import LooseVersion
+from urllib.parse import urlparse
+from urllib.error import HTTPError
+from urllib.request import Request, urlopen
+from xmlrpc.client import ServerProxy, Fault
 
 import pip
+from time import sleep
 from django.core.management.base import BaseCommand, CommandError
+from django_extensions.management.color import color_style
+from django_extensions.management.utils import signalcommand
+from pip._internal.req import InstallRequirement
+
+if LooseVersion(pip.__version__) >= LooseVersion('19.0'):
+    from pip._internal.req.constructors import install_req_from_line  # noqa
 
 try:
     try:
@@ -23,20 +34,6 @@ except ImportError:
     except ImportError:
         raise CommandError("Pip version 6 or higher is required")
 
-from django_extensions.management.color import color_style
-from django_extensions.management.utils import signalcommand
-
-try:
-    from urllib.parse import urlparse
-    from urllib.error import HTTPError
-    from urllib.request import Request, urlopen
-    from xmlrpc.client import ServerProxy
-except ImportError:
-    # Python 2
-    from urlparse import urlparse  # type: ignore
-    from urllib2 import HTTPError, Request, urlopen  # type: ignore
-    from xmlrpclib import ServerProxy  # type: ignore
-
 try:
     import requests
     HAS_REQUESTS = True
@@ -48,7 +45,7 @@ class Command(BaseCommand):
     help = "Scan pip requirement files for out-of-date packages."
 
     def add_arguments(self, parser):
-        super(Command, self).add_arguments(parser)
+        super().add_arguments(parser)
         parser.add_argument(
             "-t", "--github-api-token", action="store",
             dest="github_api_token", help="A github api authentication token."
@@ -89,7 +86,10 @@ class Command(BaseCommand):
         with PipSession() as session:
             for filename in req_files:
                 for req in parse_requirements(filename, session=session):
+                    if not isinstance(req, InstallRequirement):
+                        req = install_req_from_line(req.requirement)
                     name = req.name if req.name else req.link.filename
+
                     # url attribute changed to link in pip version 6.1.0 and above
                     if LooseVersion(pip.__version__) > LooseVersion('6.0.8'):
                         self.reqs[name] = {
@@ -146,7 +146,17 @@ class Command(BaseCommand):
             elif "dist" in req:
                 dist = req["dist"]
                 dist_version = LooseVersion(dist.version)
-                available = pypi.package_releases(req["pip_req"].name, True) or pypi.package_releases(req["pip_req"].name.replace('-', '_'), True)
+                retry = True
+                available = None
+                while retry:
+                    try:
+                        available = pypi.package_releases(req["pip_req"].name, True) or pypi.package_releases(req["pip_req"].name.replace('-', '_'), True)
+                        retry = False
+                    except Fault as err:
+                        self.stdout.write(err.faultString)
+                        self.stdout.write("Retrying in 60 seconds!")
+                        sleep(60)
+
                 available_version = self._available_version(dist_version, available)
 
                 if not available_version:
